/*
 * Copyright (c) 2003-2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: strexpdig.c,v 1.5 2005/01/27 00:29:17 ca Exp $")

#include "sm/assert.h"
#include "sm/magic.h"
#include "sm/ctype.h"
#include "sm/str-int.h"

/*
**  SM_STR_EXPDIG -- Expand %digit in string with argv[digit]
**	%[0-9] -> argv[0-9]
**	%x -> x for anything else
**
**	Parameters:
**		src -- str to expand (read)
**		dst -- str into which the expanded sequence is written (output)
**		argc -- number of valid elements in argv (0 - 10)
**		argv -- array of strings to use for expansion
**
**	Returns:
**		usual sm_ret_T
*/

sm_ret_T
sm_str_expdig(const sm_str_P src, sm_str_P dst, uint argc, sm_str_P *argv)
{
	size_t i, len;
	uint idx;
	uchar ch;
	sm_ret_T ret;

	SM_IS_BUF(dst);
	SM_IS_BUF(src);

	len = sm_str_getlen(src);
	for (i = 0; i < len; i++)
	{
		ch = sm_str_rd_elem(src, i);

		/* fixme: pass '%' in as argument?? */
		if (ch == '%' && i < len - 1)
		{
			++i;
			ch = sm_str_rd_elem(src, i);
			if (!ISDIGIT(ch))
			{
				ret = sm_str_put(dst, ch);
				if (sm_is_err(ret))
					goto error;
				continue;
			}

			idx = ch - '0';

			/*
			**  Is there an argv[] element for this?
			**  If no: %n will be replaced by an empty string!
			*/

			if (idx >= argc || argv[idx] == NULL)
				continue;
			ret = sm_str_cat(dst, argv[idx]);
			if (sm_is_err(ret))
				goto error;
			continue;
		}

		ret = sm_str_put(dst, ch);
		if (sm_is_err(ret))
			goto error;
	}

	return SM_SUCCESS;

  error:
	/* cleanup? */
	return ret;
}
