#!/bin/sh
# $Id: t-smconf-0.sh,v 1.39 2005/05/31 21:11:21 ca Exp $
# Copyright (c) 2003-2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#

# read configuration file and print it;
# uses the various modules *prtcnf.c
# Note: some of the output may depend on compile time options, e.g.
# SS_LOGDIR

test -s stop && exit 1

# source directory of this (and other) shell script
SD=`dirname $0`

ERRS=0

CNF=sm0.conf
OUT=sm0.out
OK=sm0.ok

PRG=./smconf

${PRG} -VVV |grep 'SM_USE_SASL' >${OUT} 2>&1
if test -s ${OUT}
then
AUTHOPTS='
auth {
}
'
else
AUTHOPTS=''
fi

${PRG} -VVV |grep 'SM_USE_TLS' >${OUT} 2>&1
if test -s ${OUT}
then
TLSOPTS='
tls {
}
'
else
TLSOPTS=''
fi

${PRG} -VVV |grep 'SM_USE_TLS' >${OUT} 2>&1
if test -s ${OUT}
then
TLSSRVOPTS='
tls {
  # flags to influence behavior related to STARTTLS [see documentation]
  flags={}
}
'
else
TLSSRVOPTS=''
fi

${PRG} -VVV |grep 'SM_USE_PMILTER' >${OUT} 2>&1
if test -s ${OUT}
then
PMILTEROPTS='
pmilter {
  # maximum amount of time to wait for a reply from a policy milter
  timeout=0;
}
'
else
PMILTEROPTS=""
fi

${PRG} -VVV |grep 'QMGR_TEST' >${OUT} 2>&1
if test -s ${OUT}
then
QMGRTESTOPTS='
tests=0;'
else
QMGRTESTOPTS=""
fi


cat > ${CNF} <<EOF
qmgr {
  queuereturn_timeout= 3d;
  retry_min_delay = 23m;
  retry_max_delay = 3h;
  AQ_max_entries = 8192;
  DSN_max_delay = 2s;
  smtpc_initial_connections = 19;
  smtpc_max_connections = 101;
  smtps_max_open_connections = 5;
  smtps_max_connection_rate=160;
  max_errors_per_bounce=16;
  wait_for_server = 4;
  wait_for_client = 3;
  min_disk_space = 1 MB;
  ok_disk_space = 2 MB;
	mcp_type = wait;
	user = smxq;
	restartdependencies = { smtps, smtpc, smar };
	path = "/var/spool/smx/libexec/qmgr";
	arguments = "qmgr -f smx.conf";
}

smar {
  Log_Level = 12;
  log { facility = mail; ident = smxar; }
  nameserver = {10.11.12.13, 127.0.0.1};
  dns_timeout = 6;
  address_delimiter=/;
	mcp_type = wait;
	user = smxm;
	restartdependencies = { smtps, qmgr };
	path = "/var/spool/smx/libexec/smar";
	arguments = "smar -f smx.conf";
}

smtps {
  flags = {8bitmime, access, delay_checks, };
  cdb_gid = 1200;
  daemon_address = "localhost:1234";
  IO_timeout = 60;
  wait_for_server = 4;
	port = 25;
	mcp_type = pass;
	pass_fd_socket = smtps/smtpsfd;
	user = smxs;
	path = "/var/spool/smx/libexec/smtps";
	arguments = "smtps -f smx.conf";
}

smtpc {
  remote_port = 25;
  wait_for_server = 4;
  Log_Level = 12;
  IO_timeout = 66;
	mcp_type = wait;
	user = smxc;
	path = "/var/spool/smx/libexec/smtpc";
	arguments = "smtpc -f smx.conf";
}
EOF

cat > ${OK} <<EOF
QMGR {
# Maximum number of entries in AQ (active queue)
AQ_max_entries=8192;
# Maximum number of entries in OCC (outgoing connection cache)
OCC_max_entries=0;

ibdb {
  # maximum size of each IBDB file
  size=0;
  # maximum number of open transactions in IBDB before a commit is performed
  max_open_TAs=0;
  # maximum time between commits to IBDB (micro seconds)
  max_commit_delay=0;
}

iqdb {
  # maximum number of entries in IQDB cache
  max_cache_entries=0;
  # size of hash table for IQDB
  hash_table_entries=0;
}
# minimum amount of free disk space (KB)
min_disk_space=1024;
# amount of free disk space at which normal operation continues (KB)
ok_disk_space=2048;
# initial number of outgoing connections to a single host
smtpc_initial_connections=19;
# maximum number of outgoing connections to a single host
smtpc_max_connections=101;
# maximum number of open incoming connection from a single host
smtps_max_open_connections=5;
# maximum incoming connection rate from a single host
smtps_max_connection_rate=160;
# maximum number of error messages (failed recipients) in a bounce (DSN)
max_errors_per_bounce=16;
# minimum time for retrying a delivery
retry_min_delay=1380;
# maximum time for retrying a delivery
retry_max_delay=10800;
# maximum time in queue
queuereturn_timeout=259200;
# send delay warning after this time in queue
queuedelay_timeout=0;
# timeout in address resolver
SMAR_timeout=0;
# timeout for a single delivery attempt
delivery_timeout=0;
# maximum time waiting to be scheduled
scheduler_timeout=0;
# maximum time for scheduling a DSN
DSN_max_delay=2;
# maximum amount of time to wait for a server to become available
wait_for_server=4;
# maximum amount of time to wait for a client to become available
wait_for_client=3;
log_level=0;
# Specify syslog configuration instead of logging to files

log {
}
debug_level=0;${QMGRTESTOPTS}

defedb {
  # DB page size
  page_size=0;
  # DB cache size
  cache_size=0;
  # If non-zero, a checkpoint will be done if more than the amount of KBytes of
  # log data have been written since the last checkpoint
  KBytes_written_for_checkpointing=0;
  # Minimum delay between two checkpoints
  delay_between_2_checkpoints=0;
}
# minimum number of threads [experts only]
min_threads=0;
# maximum number of threads [experts only]
max_threads=0;
# maximum number of file descriptors [experts only]
max_fds=0;
# minimum number of processes
min_processes=0;
# maximum number of processes
max_processes=0;
}
SMTPS {
# path of Unix domain socket to exchange file descriptor [use with MCP]
pass_fd_socket="smtps/smtpsfd";
# address for daemon to listen on [do not use with MCP]
daemon_address={localhost:1234, }
# log level
log_level=0;
# specify syslog(3) configuration instead of logging to files

log {
}
# group id (numeric) to use for CDB files
cdb_gid=1200;
# (unique) id for server (if multiple SMTP servers are configured)
id=0;
# maximum number of transactions per session
max_transactions=0;
# maximum number of recipients per session
max_recipients=0;
# maximum number of hops (Received: headers)
max_hops=0;
# maximum message size (KB)
max_message_size=0;
# maximum line length (0: no limit)
max_line_length=0;
# timeout for SMTP I/O operations
io_timeout=60;
# flags to influence behavior [see documentation]
flags={8bitmime, access, delay_checks, }
# maximum number of threads
max_threads=0;
# maximum number of waiting threads
max_wait_threads=0;
# minimum number of waiting threads
min_wait_threads=0;
# number of processes to start
processes=0;
# length of listen(2) queue
listen_queue=0;
# maximum amount of time to wait for a server to become available
wait_for_server=4;
# maximum amount of time to wait for a reply from smar
wait_for_smar=0;
# debug level
debug_level=0;
${TLSSRVOPTS}${AUTHOPTS}${PMILTEROPTS}# port to listen on
port=0;
# bind to this IPv4 address
address=0.0.0.0;
# minimum number of processes
min_processes=0;
# maximum number of processes
max_processes=0;
}
SMTPC {
# port to connect to on server
remote_port=25;
# number of processes to start [currently not implemented]
processes=0;
# maximum number of threads
max_threads=0;
# maximum number of waiting threads
max_wait_threads=0;
# minimum number of waiting threads
min_wait_threads=0;
flags=0;
log_level=12;

log {
}
debug_level=0;
# timeout for SMTP I/O operations
io_timeout=66;
# maximum amount of time to wait for a server to become available
wait_for_server=4;
# unique id
id=0;
# connect only to this IPv4 address [testing]
connect_only_to=0.0.0.0;
${TLSOPTS}# minimum number of processes
min_processes=0;
# maximum number of processes
max_processes=0;
}
SMAR {
# List of nameservers (IPv4 addresses)
nameserver={10.11.12.13, 127.0.0.1, }
DNS_timeout=6;
log_level=12;
# Specify syslog configuration instead of logging to files

log {
  # facility for openlog(3)
  facility=MAIL;
  # ident for openlog(3)
  ident="smxar";
}
# minimum number of threads [experts only]
min_threads=0;
# maximum number of threads [experts only]
max_threads=0;
# maximum number of file descriptors [experts only]
max_fds=0;
# DNS based blacklists
# delimiter for address extensions in local part
address_delimiter="/";
# minimum number of processes
min_processes=0;
# maximum number of processes
max_processes=0;
}
EOF

if ${PRG} ${CNF} >${OUT}
then
  if diff ${OK} ${OUT}
  then
    :
  else
    ERRS=`expr ${ERRS} + 1 `
    test X"${STOPONERROR}" != X && exit 1
  fi
else
  ERRS=`expr ${ERRS} + 1 `
  test X"${STOPONERROR}" != X && exit 1
fi

# ----------------
if test "${ERRS}" = "0"
then
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
